#!/usr/bin/env python3

import errno, json

class Errors(object):

    def __init__(self):
        return

    def __str__(self):
        return json.dumps(errno.errorcode,indent=4)
        
def main():
    errors = Errors()
    print errors
    return
    
if __name__ == '__main__': main()



#!/usr/bin/env python3

import os, re, sys, requests, hashlib, shutil, io, urllib3, logging, arrow

from urllib.parse import quote_plus, urlparse
from bs4 import BeautifulSoup as BS

from Baubles.Logger import Logger

logger = Logger()
logger.setLevel(logging.INFO)

class Downloader(object):
	
	def __init__(self, cacheDir='.cache', fresh=False):
		#urllib3.disable_warnings()
		self.cacheDir = cacheDir
		self.fresh = fresh
		logger.debug('cacheDir='+self.cacheDir)
		
	def download(self, url, file=None):
		'''
		read from disk before downloading
		'''
		logger.debug('url='+url)
		
		if not file:
			parts = urlparse(url)
			
			query = ''
			if len(parts.query):
				logger.debug('query=%s'%parts.query)
				md = hashlib.md5()
				md.update(quote_plus(parts.query).encode('utf8'))
				query = '.%s'%md.hexdigest()
				
			file = '%s/%s%s%s'%(
				self.cacheDir,
				parts.netloc,
				parts.path,
				query
			)

		logger.debug('file='+file)
	
		dirName = os.path.dirname(file)
		if dirName and not os.path.isdir(dirName):
			os.makedirs(dirName)

		data = None

		if self.fresh or not os.path.isfile(file):
			logger.info('to cache '+file)
			response = requests.get(url, verify=True, stream=True)
			if response.status_code != 200:
				file = '%s=%s'%(file,response.status_code)
				logger.warning('url=%s, code=%s'%(url, response.status_code))
			with open(file,'wb') as output:
				response.raw.decode_content = True
				shutil.copyfileobj(response.raw, output)

		else:
			logger.info('from cache '+file)
		
		with open(file, 'rb') as input:
			data = io.BytesIO(input.read())
			
		return data.getvalue()


def main():
	downloader = Downloader(fresh=False)
	now = arrow.now()
	url = 'https://docs.python.org/3/library/hashlib.html' #?_=%s'%now
	html = downloader.download(url)
	bs = BS(html, 'html5lib')
	print('\n%s'%bs.find('title').text)
	#print(html)
	#s = str(html)
	#print('\n'.join(s.split('\\n')[:5]))

		
if __name__ == '__main__': main()




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

# https://www.google.com.au/maps/@-33.8317302,151.1956613,15z

# https://googlemaps.github.io/google-maps-services-python/docs

import os,sys,re,logging,json,googlemaps

from Argumental.Argue import Argue
from Perdy.pretty import prettyPrint
from Spanners.Squirrel import Squirrel

args = Argue()
squirrel = Squirrel()

for name in logging.Logger.manager.loggerDict.keys():
    logging.getLogger(name).setLevel(logging.ERROR)

#____________________________________________________________
@args.command(single=True)
class GoogleMaps(object):

    @args.property(short='u', default='david.edson@gmail.com')
    def username(self): return
    
    def __init__(self):
        key = squirrel.get('googlemaps:%s'%self.username)
        self.gmaps = googlemaps.Client(key=key)

    @args.operation
    @args.parameter(name='name', help='name to search')
    def places(self, name):
        return self.gmaps.places(query=name)

    @args.operation
    @args.parameter(name='id', help='place id')
    def place(self, id):
        return self.gmaps.place(place_id=id)

#____________________________________________________________
if __name__ == '__main__':
    #args.parse(['places','stibo systems australia'])
    results = args.execute()
    if results:
        print(json.dumps(results, indent=4))
        



#!/usr/bin/env python3

import os,re,sys

_region = 'ap-southeast-2'
def region():
    '''
    the AWS credstash region
    '''
    return _region

_table = 'credential-store'
def table():
    '''
    the AWS DynamoDB credstash table name
    '''
    return _table

#________________________________________________________________________________________
class SquirrelBase(object):
    '''
    place holder to be substitided with the most efficient credstash implememtation available
    '''
    
    def __init__(self):
        self.values = dict()

    def get(self,name):
        raise NotImplementedError()

    def put(self,name,value):
        raise NotImplementedError()

    def delete(self,name):
        raise NotImplementedError()

    def list(self):
        raise NotImplementedError()

Squirrel = SquirrelBase

try:
    
    #____________________________________________________________________________________
    import keychain, dialogs

    class KeyChainSquirrel(SquirrelBase):
        '''
        pythonista usage of the credstash API
        '''

        def __init__(self):
            super(KeyChainSquirrel,self).__init__()

        def get(self,name):
            if name in self.values.keys():
                return self.values[name]
            value = keychain.get_password(
                table(),
                name
            )
            if not value:
                value = dialogs.text_dialog('%s ?'%name)
                self.put(name,value)
            self.values[name] = value
            return value

        def put(self,name,value):
            self.values[name] = value   
            keychain.set_password(
                table(),
                name,
                value
            )   

        def delete(self,name):
            if name in self.values.keys():
                del self.values[name]
            keychain.delete_password(
                table(),
                name
            )
            return    

        def list(self):
            return self.values.keys()

    Squirrel = KeyChainSquirrel

except:
    
    try:
        #________________________________________________________________________________
        import credstash
        from credstash import ItemNotFound
    
        class SecretSquirrel(SquirrelBase):
            '''
            class to demonstrate the usage of the credstash API
            '''
    
            def __init__(self):
                super(SecretSquirrel,self).__init__()
    
            def get(self,name):
                if name in self.values.keys():
                    return self.values[name]
                try:
                    value = credstash.getSecret(
                        name,
                        region=region(),
                        table=table()
                    )
                    self.values[name] = value
                    return value
                except ItemNotFound:
                    sys.stderr.write('cant find credstash value for key "%s"\n'%name)
                    sys.exit(1)
    
            def put(self,name,value):
                return credstash.putSecret(
                    name,value,
                    region=region(),
                    table=table()
                )            
    
            def delete(self,name):
                return credstash.deleteSecrets(
                    name,
                    region=region(),
                    table=table()
                )            
    
            def list(self):
                values = credstash.listSecrets(
                    region=region(),
                    table=table()
                )            
                return map(lambda x:x['name'], values)
    
        Squirrel = SecretSquirrel
    
    except:
        #________________________________________________________________________________
        from subprocess import Popen, PIPE
            
        class SecretAgent(SquirrelBase):
    
            def __init__(self):
                super(SecretAgent,self).__init__()
    
            def execute(self,command):
                process = Popen('credstash --table %s --region=%s %s'%(table(), region(), command),shell=True,stdout=PIPE)
                results = process.stdout.readlines()
                del process
                results = map(lambda x: x.rstrip('\r').rstrip('\n'), results)
                return results
    
            def setup(self):
                return '\n'.join(self.execute('setup'))
    
            def get(self,name):
                if name in self.values.keys():
                    return self.values[name]
                value = '\n'.join(self.execute('get \'%s\''%name))
                self.values[name] = value
                return value
    
            def put(self,name,value):
                return '\n'.join(self.execute('put \'%s\' \'%s\''%(name,value)))
    
            def delete(self,name):
                return '\n'.join(self.execute('delete \'%s\''%name))
    
            def list(self):
                p = re.compile('^(.*\S)\s+--\sversion\s\d+$')
                items = []
                for item in self.execute('list'):
                    m = p.match(item)
                    if m:
                        items.append(m.group(1))
                return items
    
        Squirrel = SecretAgent
        



#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import io, sys, os, json, xmltodict, yaml

from collections import OrderedDict as OD
from collections import deque
from asciitree import LeftAligned
from asciitree.drawing import BoxStyle, BOX_LIGHT, BOX_BLANK
from io import StringIO

from Baubles.Colours import Colours
from Perdy.pretty import prettyPrintLn, Style
from Perdy.parser import doParse
from Argumental.Argue import Argue

args = Argue()

_colour = False


@args.argument(short='c', flag=True, help='output in colour')
def colour():
	return _colour


_ascii = False


@args.argument(short='a', flag=True, help='ascii instead of boxes')
def ascii():
	return _ascii


_oneof = dict([(x, 'input as %s' % x) for x in ['xml', 'json', 'yaml']])
_format = 'json'


@args.argument(oneof=_oneof, short=True, flag=True, default=_format)
def format():
	return _format


@args.argument(positional=True, nargs='*', metavar='file')
def files():
	return []


@args.argument(short='o')
def output():
	return


@args.argument(name='test', short='T', flag=True, help='run the test harness')
def doTest():
	return False


args.parse()

fundamentals = [str, str, int, float, bool]
colours = Colours(colour=colour())


def treeFix(node):
	if not node:
		return dict()
	if type(node) in fundamentals:
		return {''.join([colours.Purple, str(node), colours.Off]): dict()}
	if type(node) is list:
		new = OD()
		for n in range(len(node)):
			new[''.join([colours.Teal, str(n), colours.Off])] = treeFix(node[n])
		return new
	if type(node) in [dict, OD]:
		for key in list(node.keys()):
			tipe = type(node[key])
			value = treeFix(node[key])
			del node[key]
			if key[0] in ['@', '#']:
				node[''.join([colours.Green, key, colours.Off])] = value
			else:
				if tipe in fundamentals:
					parts = [colours.Green]
				else:
					parts = [colours.Teal]
				parts += [key, colours.Off]
				if format() == 'xml':
					parts = ['<'] + parts + ['>']
				node[''.join(parts)] = value
	return node


def process(input, output):
	if format() == 'xml':
		o = xmltodict.parse(input.read())
	elif format() == 'yaml':
		o = yaml.load(input)
	else:
		o = json.load(input)

	if ascii():
		tr = LeftAligned()
	else:
		tr = LeftAligned(draw=BoxStyle(gfx=BOX_LIGHT, horiz_len=1))

	output.write(tr(treeFix(o)))


def main():
	_output = sys.stdout
	if output():
		_output = open(output(), 'w')
	if len(files()) == 0:
		process(sys.stdin, _output)
	else:
		for file in files():
			with open(file) as _input:
				process(_input, _output)
	if output():
		_output.close()
	return


def test():
	global _colour
	global _format
	global _ascii

	h = '\n' + '_' * 47

	j = {
		'one': {
			'one_one': {
				'one_one': [{
					'#text': '_1_1_1'
				}, {
					'#text': '_1_1_2'
				}]
			},
			'one_two': {
				'@attr': '_1_2',
				'#text': '_1_2_1'
			}
		}
	}
	print(h)
	prettyPrintLn(j)
	print(h)

	#_ascii=True
	_colour = True
	process(StringIO(json.dumps(j)), sys.stdout)
	print(h)

	x = xmltodict.unparse(j)
	doParse(StringIO(str(x)), sys.stdout, colour=True)
	print(h)
	_format = 'xml'
	process(StringIO(str(x)), sys.stdout)
	print(h)

	sio = StringIO()
	prettyPrintLn(j, output=sio, style=Style.YAML, colour=False)
	y = sio.getvalue()
	sio.close()
	#print y
	y = y.replace('#text', '"#text"')
	y = y.replace('@attr', '"@attr"')
	#print y
	prettyPrintLn(j, output=sys.stdout, style=Style.YAML, colour=True)
	print(h)
	_format = 'yaml'
	process(StringIO(y), sys.stdout)

	return


if __name__ == '__main__':
	if doTest():
		test()
	else:
		main()





#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import os, sys, re, json, xmltodict

from Argumental.Argue import Argue
from Spanners.Squirrel import Squirrel

args = Argue()


@args.command(single=True)
class SquirrelCommand(Squirrel):
	@args.attribute(short='v', flag=True)
	def verbose(self):
		return False

	def __init__(self):
		super().__init__()
		return

	@args.operation
	def get(self, name):
		'''
        get a KMS key

        '''
		return super().get(name)  #.rstrip('\r').rstrip('\n')

	@args.operation
	def put(self, name, value):
		'''
        put a KMS name,value key

        '''
		return super().put(name, value)

	@args.operation
	def delete(self, name):
		'''
        delete a KMS name
        '''
		return super().delete(name)

	@args.operation
	def list(self):
		'''
        list the KMS names

        '''
		return list(super().list())


if __name__ == '__main__':
	result = args.execute()
	if result:
		if type(result) == list:
			print('\n'.join(result))
		else:
			print(result)




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import io, sys, os, json, xmltodict, yaml

from collections import OrderedDict as OD
from collections import deque
from asciitree import LeftAligned
from asciitree.drawing import BoxStyle, BOX_LIGHT, BOX_BLANK
from io import StringIO, IOBase

from Baubles.Colours import Colours
from Perdy.pretty import prettyPrintLn, Style
from Perdy.parser import doParse
from Argumental.Argue import Argue

args = Argue()

@args.command(single=True)
class Treeify(object):
	
	@args.property(short='c', flag=True, help='output in colour')
	def colour(self): return False
	
	
	@args.property(short='a', flag=True, help='ascii instead of boxes')
	def ascii(self): return False
	
	
	_oneof = OD([(x, 'input as %s' % x) for x in ['json', 'xml', 'yaml']])
	
	
	@args.property(oneof=_oneof, short=True, flag=True, default=list(_oneof.keys())[0])
	def format(self): return
	
	
	def __init__(self, colour=False, ascii=False):
		if colour: self.colour = True
		if ascii: self.ascii = True
		self.fundamentals = [str, str, int, float, bool]
		self.collections = [list, dict, OD]
		self.colours = Colours(colour=self.colour)
	
	
	def treeFix(self, node):
		if not node:
			return dict()
		if type(node) in self.fundamentals:
			return {''.join([self.colours.Purple, str(node), self.colours.Off]): dict()}
		if type(node) is list:
			new = OD()
			for n in range(len(node)):
				key = ''.join(['[', self.colours.Teal, str(n), self.colours.Off,']'])
				new[key] = self.treeFix(node[n])
			return new
		if type(node) in [dict, OD]:
			for key in list(node.keys()):
				tipe = type(node[key])
				value = self.treeFix(node[key])
				del node[key]
				if key[0] in ['@', '#']:
					node[''.join([self.colours.Green, key, self.colours.Off])] = value
				else:
					if tipe in self.fundamentals:
						parts = [self.colours.Green]
					else:
						parts = [self.colours.Teal]
					parts += [key, self.colours.Off]
					if self.format == 'xml':
						parts = ['<'] + parts + ['>']
					node[''.join(parts)] = value
		return node
	
	
	def process(self, input, output=sys.stdout):
		if type(input) in self.collections:
			o = input
		elif isinstance(input, IOBase) or isinstance(input, StringIO):
			input = input.read()
		if type(input) in [str]:
			if self.format == 'xml':
				o = xmltodict.parse(input)
			elif self.format == 'yaml':
				o = yaml.load(input)
			else: # == 'json'
				o = json.loads(input)
		if self.ascii:
			tr = LeftAligned()
		else:
			tr = LeftAligned(draw=BoxStyle(
				label_space=0,
				gfx=BOX_LIGHT, 
				horiz_len=1
			))
	
		output.write(tr(self.treeFix(o)))
	
	
	@args.operation
	@args.parameter(name='files', short='f', nargs='*', metavar='file')
	@args.parameter(name='output', short='o')
	def bark(self, files=[], output=None):
		_output = sys.stdout
		if output:
			_output = open(output(), 'w')
		if len(files) == 0:
			self.process(sys.stdin, _output)
		else:
			for file in files:
				with open(file) as _input:
					self.process(_input, _output)
		if output:
			_output.close()
		return
	
	
	@args.operation
	def test(self):
	
		h = '\n' + '_' * 47
	
		j = {
			'one': {
				'one_one': {
					'one_one': [{
						'#text': '_1_1_1'
					}, {
						'#text': '_1_1_2'
					}]
				},
				'one_two': {
					'@attr': '_1_2',
					'#text': '_1_2_1'
				}
			}
		}
		
		print(h)
		prettyPrintLn(j)
		print(h)
	
		f = '../test/treeify.json' 
		with open(f,'w') as output:
			json.dump(j, output)
		self.bark([f])
		print(h)
		
		#self.ascii = True
		self.colour = True
		self.process(StringIO(json.dumps(j)), sys.stdout)
		print(h)
	
		x = xmltodict.unparse(j)
		doParse(StringIO(str(x)), sys.stdout, colour=True)
		print(h)
		self.format = 'xml'
		self.process(StringIO(str(x)), sys.stdout)
		print(h)
	
		sio = StringIO()
		prettyPrintLn(j, output=sio, style=Style.YAML, colour=False)
		y = sio.getvalue()
		sio.close()
		#print y
		y = y.replace('#text', '"#text"')
		y = y.replace('@attr', '"@attr"')
		#print y
		prettyPrintLn(j, output=sys.stdout, style=Style.YAML, colour=True)
		print(h)
		self.format = 'yaml'
		self.process(StringIO(y), sys.stdout)
	
		return
	
	
if __name__ == '__main__': args.execute()






#!/usr/bin/env python3

import os, re, sys, json, uuid, hashlib, arrow

class IdentityCache(object):

	filename = '.IdentityCache.json'

	#_____________________________________________
	def __init__(self):
		if os.path.isfile(self.filename):
			with open(self.filename) as input:
				self.cache = json.load(input)
		else:
			self.cache = dict()


	#_____________________________________________
	def save(self):
		with open(self.filename, 'w') as output:
			json.dump(self.cache, output, indent=4, sort_keys=True)


	#_____________________________________________
	def hasher(self, text):
		m = hashlib.md5()
		m.update(text.encode('utf8'))
		return m.hexdigest().upper()


	#_____________________________________________
	def has(self,key):
		return key in self.cache.keys()
	

	#_____________________________________________
	def get(self, key):
		if key not in self.cache.keys():
			id = '%s'%self.hasher(key)
			self.cache[key] = id
		return self.cache[key]
		
		
def main():
	cache = IdentityCache()
	now = arrow.now()
	url = 'https://docs.python.org/3/library/hashlib.html' #?_=%s'%now
	id = cache.get(url)
	
	print(id)

		
if __name__ == '__main__': main()
